{$I graph.pas}

const _uoi_maxN = 500;
const _uoi_maxStones = 1000;
const _uoi_maxGraphs = 200;
const _uoi_maxHistory = 100;

var
    _uoi_graphs, _uoi_whereToGo, _uoi_answer, _uoi_neighborsCount, _uoi_not_answered_count: longint;
    _uoi_N, _uoi_M, _uoi_currentVertex, _uoi_markedVertex, _uoi_final_answers, _uoi_not_answered, _uoi_history_count: array[1.._uoi_maxGraphs] of longint;
    _uoi_neighbors: array of longint;
    _uoi_shuffled: array[1.._uoi_maxN] of longint;
    _uoi_edges: array[1.._uoi_maxGraphs] of array[1.._uoi_maxN] of array[1.._uoi_maxN] of longint;
    _uoi_edgesCount, _uoi_stones: array[1.._uoi_maxGraphs] of array[1.._uoi_maxN] of longint;
    _uoi_history: array[1.._uoi_maxGraphs] of array[1.._uoi_maxHistory] of array[0.._uoi_maxN + 5] of longint;

function _uoi_testAndClearHistory(g: longint): boolean;
var i, j, current, whereToGo, answer, subproblem, l: longint;
    neighbors: array of longint;
begin
    for i := _uoi_history_count[g] downto 1 do begin
        current := _uoi_history[g][i][_uoi_maxN + 2];
        whereToGo := -1;
        answer := -1;
        if _uoi_markedVertex[g] = 0 then
            subproblem := 1
        else
            subproblem := 2;
        l := _uoi_history[g][i][_uoi_maxN + 1];
        setlength(neighbors, l);
        for j := 0 to l - 1 do
            neighbors[j] := _uoi_history[g][i][j];
        graph(subproblem, l, neighbors, current, whereToGo, answer);
        if (current <> _uoi_history[g][i][_uoi_maxN + 3]) or (whereToGo <> _uoi_history[g][i][_uoi_maxN + 4]) or (answer <> _uoi_history[g][i][_uoi_maxN + 5]) then begin
            _uoi_testAndClearHistory := false;
            exit;
        end;
    end;
    _uoi_history_count[g] := 0;
    _uoi_testAndClearHistory := true;
end;

procedure _uoi_reshuffle(arraySize: longint);
var i, index, t: longint;
begin
    for i := 1 to arraySize do
        _uoi_shuffled[i] := i;
    for i := arraySize downto 2 do begin
        index := random(i) + 1;
        t := _uoi_shuffled[index];
        _uoi_shuffled[index] := _uoi_shuffled[i];
        _uoi_shuffled[i] := t;
    end;
end;

procedure _uoi_run;
var i, g, g_index, a, b, subproblem: longint;
    found: boolean;
    f: text;
begin
    assign(f, 'graph_covert_ytr.dat');
    reset(f);
    read(f, _uoi_graphs);
    for g := 1 to _uoi_graphs do begin
        read(f, _uoi_N[g], _uoi_M[g], _uoi_currentVertex[g], _uoi_markedVertex[g]);
        for i := 1 to _uoi_N[g] do begin
            _uoi_edgesCount[g][i] := 0;
            _uoi_stones[g][i] := 0;
        end;
        if _uoi_markedVertex[g] > 0 then
            _uoi_stones[g][_uoi_markedVertex[g]] := 1;
        for i := 1 to _uoi_M[g] do begin
            read(f, a, b);
            inc(_uoi_edgesCount[g][a]);
            _uoi_edges[g][a][_uoi_edgesCount[g][a]] := b;
            inc(_uoi_edgesCount[g][b]);
            _uoi_edges[g][b][_uoi_edgesCount[g][b]] := a;
        end;
    end;
    close(f);

    _uoi_not_answered_count := _uoi_graphs;
    for i := 1 to _uoi_graphs do
        _uoi_not_answered[i] := i;

    while _uoi_not_answered_count > 0 do begin
        g_index := random(_uoi_not_answered_count) + 1;
        g := _uoi_not_answered[g_index];
        if _uoi_history_count[g] = _uoi_maxHistory then
            if not _uoi_testAndClearHistory(g) then
                halt(10005);
        inc(_uoi_history_count[g]);
        _uoi_history[g][_uoi_history_count[g]][_uoi_maxN + 1] := _uoi_edgesCount[g][_uoi_currentVertex[g]];
        _uoi_reshuffle(_uoi_edgesCount[g][_uoi_currentVertex[g]]);
        setlength(_uoi_neighbors, _uoi_edgesCount[g][_uoi_currentVertex[g]]);
        for i := 1 to _uoi_edgesCount[g][_uoi_currentVertex[g]] do begin
            _uoi_neighbors[i - 1] := _uoi_stones[g][_uoi_edges[g][_uoi_currentVertex[g]][_uoi_shuffled[i]]];
            _uoi_history[g][_uoi_history_count[g]][i - 1] := _uoi_neighbors[i - 1];
        end;
        _uoi_history[g][_uoi_history_count[g]][_uoi_maxN + 2] := _uoi_stones[g][_uoi_currentVertex[g]];
        _uoi_whereToGo := -1;
        _uoi_answer := -1;
        if _uoi_markedVertex[g] = 0 then
            subproblem := 1
        else
            subproblem := 2;
        graph(subproblem, _uoi_edgesCount[g][_uoi_currentVertex[g]], _uoi_neighbors, _uoi_stones[g][_uoi_currentVertex[g]], _uoi_whereToGo, _uoi_answer);
        _uoi_history[g][_uoi_history_count[g]][_uoi_maxN + 3] := _uoi_stones[g][_uoi_currentVertex[g]];
        _uoi_history[g][_uoi_history_count[g]][_uoi_maxN + 4] := _uoi_whereToGo;
        _uoi_history[g][_uoi_history_count[g]][_uoi_maxN + 5] := _uoi_answer;
        if (_uoi_stones[g][_uoi_currentVertex[g]] < 0) or (_uoi_stones[g][_uoi_currentVertex[g]] > _uoi_maxStones) then
            halt(10001);
        if (_uoi_whereToGo <> -1) and (_uoi_answer <> -1) then
            halt(10002);
        if (_uoi_whereToGo = -1) and (_uoi_answer = -1) then
            halt(10003);
        if _uoi_answer <> -1 then begin
            if not _uoi_testAndClearHistory(g) then
                halt(10005);
            _uoi_final_answers[g] := _uoi_answer;
            _uoi_not_answered[g_index] := _uoi_not_answered[_uoi_not_answered_count];
            dec(_uoi_not_answered_count);
            continue;
        end;

        found := false;
        for i := 1 to _uoi_edgesCount[g][_uoi_currentVertex[g]] do
            if _uoi_stones[g][_uoi_edges[g][_uoi_currentVertex[g]][i]] = _uoi_whereToGo then begin
                _uoi_currentVertex[g] := _uoi_edges[g][_uoi_currentVertex[g]][i];
                found := true;
                break;
            end;
        if not found then
            halt(10004);
    end;

    assign(f, 'graph_covert_ytr.sol');
    rewrite(f);
    for i := 1 to _uoi_graphs do
        writeln(f, _uoi_final_answers[i]);
    close(f);
end;

begin
    _uoi_run();
end.